clear all
close all

addpath Preprocessing % The preprocessing files
addpath TV

% Method specification -- 1=TV-M,2=TV-m(G.f),3=TV-u,4=TV-u/ME
METHOD=4;

% Load Data : 
load('Beethoven.mat')

% Preprocess the images using the method described in Wu et al 2010 paper
I = inexact_alm_rpca(I,3/sqrt(length(find(mask>0))),1e-7,1000);		

% Solve the UPS problem using TV (see JMIV 2014 paper)
[N,RHO,L_hat] = ups_tv(I,mask,METHOD);

% Show albedo map
figure(1)
imagesc(RHO)
colormap gray
axis off
axis equal

% Integrate the normal field and display
p=reshape(-N(:,2)./(N(:,3)+eps),size(mask));
q=reshape(N(:,1)./(N(:,3)+eps),size(mask));
U = direct_weighted_poisson(p,q,mask);
figure(2)
surfl(U,[180 90])
axis equal
axis off
shading flat
colormap gray
view(130,15)
%~ camlight headlight

% Compute calibrated normals
N_GT=I*pinv(L);
n_GT=N_GT./repmat(sqrt(sum(N_GT.^2,2)),[1 3]);

% Compute Mean Angular Error on normals
inner1=sum(N.*n_GT,2);
inner2=sum((N*[-1 0 0;0 -1 0; 0 0 1]).*n_GT,2);
inner3=sum((N*[-1 0 0;0 1 0; 0 0 1]).*n_GT,2);
inner4=sum((N*[1 0 0;0 -1 0; 0 0 1]).*n_GT,2);
angle1=acosd(abs(inner1(mask>0)));
angle2=acosd(abs(inner2(mask>0)));
angle3=acosd(abs(inner3(mask>0)));
angle4=acosd(abs(inner4(mask>0)));
mean_angle1=mean(angle1);
mean_angle2=mean(angle2);	
mean_angle3=mean(angle3);	
mean_angle4=mean(angle4);	
MAE=min([mean_angle1 mean_angle2 mean_angle3 mean_angle4])
	
